# -*- coding: utf-8 -*-

import os

PICTURES_PATH = os.path.dirname(__file__)

# BACKGROUND_COLOR = (13, 13, 13)
BACKGROUND_IMAGE = 'background.jpg'
METER_1_COLOR = (80, 80, 80)
METER_2_COLOR = (70, 70, 90)

FIELD_WIDTH = 1200
FIELD_HEIGHT = 600

TEAMS_COUNT = 4

DEBUG = False

MAX_HEALTH = 100
STING_POWER = 50
HEALTH_TOP_UP_SPEED = 0.5
BEEHIVE_SAFE_DISTANCE = 200

# See robogame_engine.constants
