# -*- coding: utf-8 -*-

import os

PICTURES_PATH = os.path.dirname(__file__)

BACKGROUND_COLOR = (85, 107, 47)

FIELD_WIDTH = 1200
FIELD_HEIGHT = 600

METER_1_COLOR = (0, 255, 70)
METER_2_COLOR = (232, 129, 31)

TEAMS_COUNT = 4

DEBUG = False

MAX_HEALTH = 100
STING_POWER = 50
HEALTH_TOP_UP_SPEED = 0.5
BEEHIVE_SAFE_DISTANCE = 200

# See robogame_engine.constants
